clear all;
clc;
close all;

%% UCA settings
Nt = 800;
fc = 30e9;
c = 3e8;
lambda = c/fc;
d = lambda/2;

r_radius = Nt*d/2/pi; % half-wavelength spacing
D = 2*(2*r_radius)^2/lambda;

%% beamforming in distance UCA
theta_0 = 0;
r_0 = 100000;
at = generate_UCA(fc, Nt, r_radius, r_0, theta_0);

num_bessel = 10000;
r_list = linspace(5, 100, num_bessel);
g_r = zeros(1, num_bessel);
g_r_est = zeros(1, num_bessel);
for idx_r = 1:num_bessel
    r_iter = r_list(idx_r);
    h0 = generate_UCA(fc, Nt, r_radius, r_iter, theta_0);
    g_r(idx_r)=  abs(at*h0');
    
    xi = -2*pi*r_radius^2/lambda/4*(1/(r_iter)-1/r_0);
    g_r_est(idx_r) = abs(besselj(0, xi));
end


%% beamforming in distance ULA
Nt_ula = floor(Nt/pi);
g_ula = zeros(1,num_bessel);
at_ula = near_field_manifold(Nt_ula, d, fc, r_0, theta_0);
for i_r = 1:num_bessel
    r = r_list(i_r);
    at_ula2 = near_field_manifold(Nt_ula, d, fc, r, theta_0);
    g_ula(i_r) = abs(at_ula*at_ula2');
end

figure;
hold on;
plot(r_list, g_r,'r-','LineWidth',1.5);
plot(r_list, g_ula,'b-','LineWidth',1.5);
grid on;
box on;
xlabel('Distance');
ylabel('Beamforming Gain');
legend('UCA','ULA','Location','northwest');

figure;
hold on;
plot(r_list, g_r,'r-','LineWidth',1.5);
plot(r_list, g_r_est,'g--','LineWidth',1.5);
plot(r_list, g_ula,'b-','LineWidth',1.5);
grid on;
box on;
xlabel('Distance');
ylabel('Beamforming Gain');
legend('Calculated UCA','Estimated  UCA','Calculated ULA','Location','northwest');

%% effective rayleigh distance for different angles
threshold = 0.95;
num_bessel = 3000;

num_phi = 10000;
phi_list = linspace(-pi/2, pi/2, num_phi);
g = zeros(1, num_phi);
r_uca = zeros(1, num_phi);
r_ula = zeros(1, num_phi);
% epsilon = 0.6406;

% decide the value for UCA
r_list = linspace(0, 1, num_bessel);
g_r = abs(besselj(0, r_list));
[~,idx] = min(abs(g_r-threshold));
epsilon_c = r_list(idx);
epsilon_c_real = pi/8/epsilon_c;

%decide the epsilon value for ULA
num_point = 100000;
beta = linspace(0.1, 1, num_bessel);
[f1_sin, f1_cos] = fresnel(beta, num_point);
g_r_ula = abs(f1_cos+1i*f1_sin)./beta;

[~,idx] = min(abs(g_r_ula-threshold));
epsilon_l = beta(idx);
for i_phi = 1:num_phi
    phi = phi_list(i_phi);
    r_uca(i_phi) = pi*r_radius^2/2/lambda/epsilon_c;
    r_ula(i_phi) = (2*r_radius)^2*cos(phi)^2/2/epsilon_l^2/lambda;
end
figure;
hold on;
h1 = plot(phi_list, ones(1, num_phi),'k--','Linewidth',1.5);
h2 = plot(phi_list, r_ula./r_uca,'r-','Linewidth',1.5);
axis([min(phi_list) max(phi_list) 0 1.1]);
grid on;
box on;
xlabel('\phi');
ylabel('Ratio value');
legend([h2], {'\rho(\phi)'},'FontSize',12);

%% calculated effective rayleigh distance UCA
num_r = 1600;
num_theta = Nt/2;

r_list = linspace(3, D, num_r);
theta_list = linspace(-pi/2, pi/2, num_theta);
x_list = r_list.' * cos(theta_list);
y_list = r_list.' * sin(theta_list);

g = zeros(num_r, num_theta);
r0 = 100000;
for idx_theta = 1:1 % calculate one angle for simplicity
    theta0 = theta_list(idx_theta);
    h0 = generate_UCA(fc, Nt, r_radius, r0, theta0);
    for idx_r = 1:num_r
        h_grid = generate_UCA(fc, Nt, r_radius, r_list(idx_r), theta_list(idx_theta));
        g(idx_r, :)=  abs(h_grid*h0');
    end
end

g_over = (g>threshold);
[~, idx] = max(g_over,[], 1);
g_r_list = r_list(idx);
[x_uca_cal, y_uca_cal] = pol2cart(theta_list, g_r_list);

%% calculated effective rayleigh distance ULA
Nt_ula = floor(r_radius*2/d);
D_ula = 2*(Nt_ula*d)^2/lambda;
num_r = 800;
num_theta = Nt_ula;

r_list = linspace(3, D_ula, num_r);
theta_list = linspace(-pi/6, pi/6, num_theta);
x_list = r_list.' * cos(theta_list);
y_list = r_list.' * sin(theta_list);

g_ula = zeros(num_r, num_theta);
r_center = 10000;
r0 = r_center;
for idx_theta = 1:num_theta
    theta0 = theta_list(idx_theta);
    h0 = near_field_manifold(Nt_ula, d, fc, r0, theta0);
    for idx_r = 1:num_r
        h_grid = near_field_manifold(Nt_ula, d, fc, r_list(idx_r), theta_list(idx_theta));
        g_ula(idx_r, idx_theta)=  abs(h_grid*h0');
    end
end

g_ula_over = (g_ula>threshold);
[~, idx] = max(g_ula_over,[], 1);
g_ula_r_list = r_list(idx);
r_const = mean(g_ula_r_list./cos(theta_list).^2);
theta_plot = linspace(-pi/2, pi/2, 1000);
r_ula_plot = r_const*cos(theta_plot).^2;
[x_ula_cal, y_ula_cal] = pol2cart(theta_plot, r_ula_plot);


%% plot in 2D space
[x_ula,y_ula] = pol2cart(phi_list, r_ula);
[x_uca,y_uca] = pol2cart(phi_list, r_uca);

figure;
hold on;
plot(x_uca_cal,y_uca_cal,'b-','Linewidth',1.5)
plot(x_ula_cal,y_ula_cal,'r-','Linewidth',1.5)
xlabel('x-axis');
ylabel('y-axis');
legend('ERD UCA','ERD ULA');
grid on;
box on;


figure;
hold on;
plot(x_uca_cal,y_uca_cal,'b-','Linewidth',1.5)
plot(x_uca,y_uca,'g--','Linewidth',1.5)
plot(x_ula_cal,y_ula_cal,'r-','Linewidth',1.5)
plot(x_ula,y_ula,'k--','Linewidth',1.5)
xlabel('x-axis');
ylabel('y-axis');
legend('Calculated ERD UCA','Estimated ERD UCA',...
    'Calculated ERD UCA','Estimated ERD UCA');
grid on;
box on;



